@@ -84,9 +84,12 @@ class DeploymentConfigParameters:
     cfg_dict: dict[str, t.Any] | None = None
     _param_config: dict[str, t.Any] | None = None
 
-    def verify(self, create: bool = True):
-        from bentoml._internal.configuration.containers import BentoMLContainer
-
+    def verify(
+        self,
+        create: bool = True,
+        *,
+        _client: RestApiClient,
+    ):
         from .secret import SecretAPI
 
         if self.config_dict:
@@ -152,14 +155,21 @@ class DeploymentConfigParameters:
                 if self.cli:
                     rich.print(f"building bento from [green]{bento_name}[/] ...")
                 bento_info = ensure_bento(
-                    project_path=bento_name, bare=self.dev, cli=self.cli
+                    project_path=bento_name,
+                    bare=self.dev,
+                    cli=self.cli,
+                    _client=_client,
                 )
             elif self.dev:  # dev mode and bento is built
                 return
             else:
                 if self.cli:
                     rich.print(f"using bento [green]{bento_name}[/]...")
-                bento_info = ensure_bento(bento=str(bento_name), cli=self.cli)
+                bento_info = ensure_bento(
+                    bento=str(bento_name),
+                    cli=self.cli,
+                    _client=_client,
+                )
             if create:
                 manifest = (
                     bento_info.get_manifest()
@@ -169,7 +179,7 @@ class DeploymentConfigParameters:
                 required_envs = [env.name for env in manifest.envs if not env.value]
                 provided_envs: list[str] = [env["name"] for env in (self.envs or [])]
                 if self.secrets:
-                    secret_api = SecretAPI(BentoMLContainer.rest_api_client.get())
+                    secret_api = SecretAPI(_client)
                     for secret_name in self.secrets:
                         secret = secret_api.get(secret_name, cluster=self.cluster)
                         if secret.content.type == "env":
@@ -205,7 +215,12 @@ class DeploymentConfigParameters:
         else:
             return self.cfg_dict.get("cluster")
 
-    def get_config_dict(self, bento: str | None = None):
+    def get_config_dict(
+        self,
+        bento: str | None = None,
+        *,
+        _client: RestApiClient,
+    ):
         if self.cfg_dict is None:
             raise BentoMLException(
                 "DeploymentConfigParameters.verify() must be called first"
@@ -216,7 +231,7 @@ class DeploymentConfigParameters:
                     raise BentoMLException("Bento is required")
                 bento = self.cfg_dict.get("bento")
 
-            info = ensure_bento(bento=bento)
+            info = ensure_bento(bento=bento, _client=_client)
             if info.entry_service == "":
                 # for compatibility
                 self.service_name = "apiserver"
@@ -951,7 +966,7 @@ class Deployment:
                         cli=False,
                         dev=True,
                     )
-                    update_config.verify(create=False)
+                    update_config.verify(create=False, _client=self._client)
                     self = deployment_api.update(update_config)
                     target = self._refetch_target(False)
                     needs_update = False
@@ -1341,7 +1356,7 @@ class DeploymentAPI:
         ):
             raise ValueError("bento is required")
 
-        config_params = deployment_config_params.get_config_dict()
+        config_params = deployment_config_params.get_config_dict(_client=rest_client)
         config_struct = bentoml_cattr.structure(config_params, CreateDeploymentSchemaV2)
         self._fix_and_validate_schema(config_struct)
 
@@ -1366,6 +1381,7 @@ class DeploymentAPI:
         Returns:
             The DeploymentInfo object.
         """
+        rest_client = self._client
         name = deployment_config_params.get_name()
         if name is None:
             raise ValueError("name is required")
@@ -1376,6 +1392,7 @@ class DeploymentAPI:
 
         config_params = deployment_config_params.get_config_dict(
             orig_dict.get("bento"),
+            _client=rest_client,
         )
         deep_merge(orig_dict, config_params)
         config_struct = bentoml_cattr.structure(orig_dict, UpdateDeploymentSchemaV2)
@@ -1431,7 +1448,8 @@ class DeploymentAPI:
                     f"Deployment cluster cannot be changed, current cluster is {deployment_schema.cluster.name}"
                 )
             config_struct = bentoml_cattr.structure(
-                deployment_config_params.get_config_dict(), UpdateDeploymentSchemaV2
+                deployment_config_params.get_config_dict(_client=rest_client),
+                UpdateDeploymentSchemaV2,
             )
             self._fix_and_validate_schema(config_struct)
 
