@@ -1,8 +1,9 @@
 from __future__ import annotations
 
-import typing as t
-import logging
+import importlib.metadata
 import importlib.util
+import logging
+import typing as t
 from typing import TYPE_CHECKING
 
 from bentoml.exceptions import BentoMLException
@@ -10,11 +11,6 @@ from bentoml.exceptions import BentoMLException
 from ...types import LazyType
 from ...utils.lazy_loader import LazyLoader
 
-try:
-    import importlib.metadata as importlib_metadata
-except ImportError:
-    import importlib_metadata
-
 if TYPE_CHECKING:
     import tensorflow as tf
 
@@ -99,9 +95,9 @@ def get_tf_version() -> str:
         # For the metadata, we have to look for both tensorflow and tensorflow-cpu
         for pkg in candidates:
             try:
-                _tf_version = importlib_metadata.version(pkg)
+                _tf_version = importlib.metadata.version(pkg)
                 break
-            except importlib_metadata.PackageNotFoundError:
+            except importlib.metadata.PackageNotFoundError:
                 pass
     return _tf_version
 
@@ -337,7 +333,7 @@ def _pretty_format_function_call(base: str, name: str, arg_names: t.Tuple[t.Any]
 
 def _pretty_format_positional(positional: t.Optional["tf_ext.TensorSignature"]) -> str:
     if positional is not None:
-        return f'Positional arguments ({len(positional)} total):\n    {"    * ".join(str(a) for a in positional)}'  # noqa
+        return f"Positional arguments ({len(positional)} total):\n    {'    * '.join(str(a) for a in positional)}"  # noqa
     return "No positional arguments.\n"
 
 
@@ -406,9 +402,7 @@ def cast_tensor_by_spec(
         _input
     ) or LazyType["tf_ext.CastableTensorType"](
         "tensorflow.python.framework.ops.EagerTensor"
-    ).isinstance(
-        _input
-    ):
+    ).isinstance(_input):
         # TensorFlow Issues #43038
         # pylint: disable=unexpected-keyword-arg, no-value-for-parameter
         return t.cast(
@@ -460,7 +454,8 @@ class tf_function_wrapper:  # pragma: no cover
         # how signature with kwargs works?
         # https://github.com/tensorflow/tensorflow/blob/v2.0.0/tensorflow/python/eager/function.py#L1519
         transformed_args: t.Tuple[t.Any, ...] = tuple(
-            cast_tensor_by_spec(arg, spec) for arg, spec in zip(args, self.arg_specs)  # type: ignore[arg-type]
+            cast_tensor_by_spec(arg, spec)
+            for arg, spec in zip(args, self.arg_specs)  # type: ignore[arg-type]
         )
 
         transformed_kwargs = {
