@@ -1,29 +1,29 @@
 from __future__ import annotations
 
+import functools
 import io
+import logging
 import os
 import typing as t
-import logging
-import functools
-from enum import Enum
 from concurrent.futures import ThreadPoolExecutor
+from enum import Enum
 
 from starlette.requests import Request
 from starlette.responses import Response
 
-from .base import IODescriptor
-from ..types import LazyType
-from ..utils.pkg import find_spec
-from ..utils.http import set_cookies
 from ...exceptions import BadInput
 from ...exceptions import InvalidArgument
-from ...exceptions import UnprocessableEntity
 from ...exceptions import MissingDependencyException
+from ...exceptions import UnprocessableEntity
 from ...grpc.utils import import_generated_stubs
 from ..service.openapi import SUCCESS_DESCRIPTION
-from ..utils.lazy_loader import LazyLoader
-from ..service.openapi.specification import Schema
 from ..service.openapi.specification import MediaType
+from ..service.openapi.specification import Schema
+from ..types import LazyType
+from ..utils.http import set_cookies
+from ..utils.lazy_loader import LazyLoader
+from ..utils.pkg import find_spec
+from .base import IODescriptor
 
 EXC_MSG = "pandas' is required to use PandasDataFrame or PandasSeries. Install with 'pip install bentoml[io-pandas]'"
 
@@ -38,8 +38,8 @@ if t.TYPE_CHECKING:
     from bentoml.grpc.v1alpha1 import service_pb2 as pb_v1alpha1
 
     from .. import external_typing as ext
+    from ..context import ServiceContext as Context
     from .base import OpenAPIResponse
-    from ..context import InferenceApiContext as Context
 
 else:
     pb, _ = import_generated_stubs("v1")
@@ -66,15 +66,15 @@ def get_parquet_engine() -> str:
         )
 
 
-def _openapi_types(item: str) -> str:  # pragma: no cover
+def _openapi_types(item: t.Any) -> str:  # pragma: no cover
     # convert pandas types to OpenAPI types
-    if item.startswith("int"):
+    if pd.api.types.is_integer_dtype(item):
         return "integer"
-    elif item.startswith("float") or item.startswith("double"):
+    elif pd.api.types.is_float_dtype(item):
         return "number"
-    elif item.startswith("str") or item.startswith("date"):
+    elif pd.api.types.is_string_dtype(item) or pd.api.types.is_datetime64_dtype(item):
         return "string"
-    elif item.startswith("bool"):
+    elif pd.api.types.is_bool_dtype(item):
         return "boolean"
     else:
         return "object"
@@ -143,7 +143,7 @@ def _series_openapi_schema(
 
 class SerializationFormat(Enum):
     JSON = "application/json"
-    PARQUET = "application/octet-stream"
+    PARQUET = "application/vnd.apache.parquet"
     CSV = "text/csv"
 
     def __init__(self, mime_type: str):
@@ -169,7 +169,8 @@ def _infer_serialization_format_from_request(
 
     if content_type == "application/json":
         return SerializationFormat.JSON
-    elif content_type == "application/octet-stream":
+    # We accept octet-stream as Parquet to avoid breaking legacy code.
+    elif content_type in ("application/vnd.apache.parquet", "application/octet-stream"):
         return SerializationFormat.PARQUET
     elif content_type == "text/csv":
         return SerializationFormat.CSV
@@ -224,7 +225,7 @@ class PandasDataFrame(
 
        runner = bentoml.sklearn.get("sklearn_model_clf").to_runner()
 
-       svc = bentoml.Service("iris-classifier", runners=[runner])
+       svc = bentoml.legacy.Service("iris-classifier", runners=[runner])
 
        @svc.api(input=input_spec, output=PandasDataFrame())
        def predict(input_arr):
@@ -309,7 +310,7 @@ class PandasDataFrame(
                         It is also the serialization format used for the response. Possible values are:
 
                         - :obj:`json` - JSON text format (inferred from content-type ``"application/json"``)
-                        - :obj:`parquet` - Parquet binary format (inferred from content-type ``"application/octet-stream"``)
+                        - :obj:`parquet` - Parquet binary format (inferred from content-type ``"application/vnd.apache.parquet"``)
                         - :obj:`csv` - CSV text format (inferred from content-type ``"text/csv"``)
 
     Returns:
@@ -369,7 +370,7 @@ class PandasDataFrame(
                             It is also the serialization format used for the response. Possible values are:
 
                             - :obj:`json` - JSON text format (inferred from content-type ``"application/json"``)
-                            - :obj:`parquet` - Parquet binary format (inferred from content-type ``"application/octet-stream"``)
+                            - :obj:`parquet` - Parquet binary format (inferred from content-type ``"application/vnd.apache.parquet"``)
                             - :obj:`csv` - CSV text format (inferred from content-type ``"text/csv"``)
 
         Returns:
@@ -442,6 +443,8 @@ class PandasDataFrame(
             return str(value.dtype)
         elif isinstance(value, bool):
             return str(value)
+        elif isinstance(value, str):
+            return value
         elif isinstance(value, dict):
             return {str(k): self._convert_dtype(v) for k, v in value.items()}
         elif value is None:
@@ -667,30 +670,25 @@ class PandasDataFrame(
                 # large tabular data
                 if len(content.ListFields()) != 1:
                     raise BadInput(
-                        f"Array contents can only be one of given values key. Use one of '{list(map(lambda f: f[0].name,content.ListFields()))}' instead."
+                        f"Array contents can only be one of given values key. Use one of '{list(map(lambda f: f[0].name, content.ListFields()))}' instead."
                     ) from None
                 return {str(i): c for i, c in enumerate(content.ListFields()[0][1])}
 
             with ThreadPoolExecutor(max_workers=10) as executor:
                 futures = executor.map(process_columns_contents, field.columns)
                 data.extend([i for i in list(futures)])
-            dataframe = pd.DataFrame(
-                dict(zip(field.column_names, data)),
-                columns=t.cast(t.List[str], field.column_names),
-            )
+            dataframe = pd.DataFrame(dict(zip(field.column_names, data)))
         return self.validate_dataframe(dataframe)
 
     @t.overload
     async def _to_proto_impl(
         self, obj: ext.PdDataFrame, *, version: t.Literal["v1"]
-    ) -> pb.DataFrame:
-        ...
+    ) -> pb.DataFrame: ...
 
     @t.overload
     async def _to_proto_impl(
         self, obj: ext.PdDataFrame, *, version: t.Literal["v1alpha1"]
-    ) -> pb_v1alpha1.DataFrame:
-        ...
+    ) -> pb_v1alpha1.DataFrame: ...
 
     async def _to_proto_impl(
         self, obj: ext.PdDataFrame, *, version: str
@@ -749,9 +747,9 @@ class PandasDataFrame(
         return pyarrow.RecordBatch.from_pandas(df)
 
     def spark_schema(self) -> pyspark.sql.types.StructType:
-        from pyspark.sql.types import StructType
-        from pyspark.sql.types import StructField
         from pyspark.pandas.typedef import as_spark_type
+        from pyspark.sql.types import StructField
+        from pyspark.sql.types import StructType
 
         if self._dtype is None or self._dtype:
             raise InvalidArgument(
@@ -802,7 +800,7 @@ class PandasSeries(
 
         runner = bentoml.sklearn.get("sklearn_model_clf").to_runner()
 
-        svc = bentoml.Service("iris-classifier", runners=[runner])
+        svc = bentoml.legacy.Service("iris-classifier", runners=[runner])
 
         @svc.api(input=PandasSeries(), output=PandasSeries())
         def predict(input_arr):
@@ -950,6 +948,10 @@ class PandasSeries(
         # TODO: support extension dtypes
         if LazyType["ext.NpNDArray"]("numpy", "ndarray").isinstance(value):
             return str(value.dtype)
+        elif isinstance(value, np.dtype):
+            return str(value)
+        elif isinstance(value, str):
+            return value
         elif isinstance(value, bool):
             return str(value)
         elif isinstance(value, dict):
@@ -1140,14 +1142,12 @@ class PandasSeries(
     @t.overload
     async def _to_proto_impl(
         self, obj: ext.PdSeries, *, version: t.Literal["v1"]
-    ) -> pb.Series:
-        ...
+    ) -> pb.Series: ...
 
     @t.overload
     async def _to_proto_impl(
         self, obj: ext.PdSeries, *, version: t.Literal["v1alpha1"]
-    ) -> pb_v1alpha1.Series:
-        ...
+    ) -> pb_v1alpha1.Series: ...
 
     async def _to_proto_impl(
         self, obj: ext.PdSeries, *, version: str
@@ -1180,7 +1180,7 @@ class PandasSeries(
             ) from None
         try:
             fieldpb = npdtype_to_fieldpb_map()[obj.dtype]
-            return pb.Series(**{fieldpb: obj.ravel().tolist()})
+            return pb.Series(**{fieldpb: obj.tolist()})
         except KeyError:
             raise InvalidArgument(
                 f"Unsupported dtype '{obj.dtype}' for response message."
@@ -1205,9 +1205,9 @@ class PandasSeries(
         return pyarrow.RecordBatch.from_pandas(df)
 
     def spark_schema(self) -> pyspark.sql.types.StructType:
-        from pyspark.sql.types import StructType
-        from pyspark.sql.types import StructField
         from pyspark.pandas.typedef import as_spark_type
+        from pyspark.sql.types import StructField
+        from pyspark.sql.types import StructType
 
         if self._dtype is None or self._dtype is True:
             raise InvalidArgument(
