@@ -1,27 +1,23 @@
 from __future__ import annotations
 
+import asyncio
+import collections
+import functools
+import logging
 import time
 import typing as t
-import asyncio
-import logging
-import functools
-import traceback
-import collections
+from functools import cached_property
 
 import attr
 import numpy as np
+from starlette.concurrency import run_in_threadpool
 
-from ..utils import cached_property
+from ..utils import is_async_callable
 from ..utils.alg import TokenBucket
 
 logger = logging.getLogger(__name__)
 
 
-if t.TYPE_CHECKING:
-    from ..runner.utils import Params
-    from ..runner.container import Payload
-
-
 class NonBlockSema:
     def __init__(self, count: int):
         self.sema = count
@@ -39,14 +35,6 @@ class NonBlockSema:
         self.sema += 1
 
 
-@attr.define
-class Job:
-    enqueue_time: float
-    data: Params[Payload]
-    future: asyncio.Future[t.Any]
-    dispatch_time: float = 0
-
-
 class Optimizer:
     """
     Analyse historical data to optimize CorkDispatcher.
@@ -67,7 +55,7 @@ class Optimizer:
         self.o_a = min(2, max_latency * 2.0 / 30)
         self.o_b = min(1, max_latency * 1.0 / 30)
 
-        self.wait = 0.01  # the avg wait time before outbound called
+        self.wait = 0  # the avg wait time before outbound called
 
         self._refresh_tb = TokenBucket(2)  # to limit params refresh interval
         self.outbound_counter = 0
@@ -85,6 +73,12 @@ class Optimizer:
             self.trigger_refresh()
 
     def trigger_refresh(self):
+        if not self.o_stat:
+            logger.debug(
+                "o_stat is empty, skip dynamic batching optimizer params update"
+            )
+            return
+
         x = tuple((i, 1) for i, _, _ in self.o_stat)
         y = tuple(i for _, i, _ in self.o_stat)
 
@@ -106,7 +100,15 @@ T_IN = t.TypeVar("T_IN")
 T_OUT = t.TypeVar("T_OUT")
 
 
-class CorkDispatcher:
+@attr.define
+class Job(t.Generic[T_IN, T_OUT]):
+    enqueue_time: float
+    data: T_IN
+    future: asyncio.Future[T_OUT | Exception]
+    dispatch_time: float = 0
+
+
+class CorkDispatcher(t.Generic[T_IN, T_OUT]):
     """
     A decorator that:
         * wrap batch function
@@ -114,50 +116,58 @@ class CorkDispatcher:
     The wrapped function should be an async function.
     """
 
+    callback: t.Callable[[t.Sequence[T_IN]], t.Coroutine[None, None, t.Sequence[T_OUT]]]
+    # the interval between each poll
+    TICKET_INTERVAL = 0.001
+
     def __init__(
         self,
         max_latency_in_ms: int,
         max_batch_size: int,
+        *,
         shared_sema: t.Optional[NonBlockSema] = None,
-        fallback: t.Callable[[], t.Any] | type[t.Any] | None = None,
-    ):
+        fallback: t.Callable[[], T_OUT],
+        get_batch_size: t.Callable[[T_IN], int] = lambda x: x.sample.batch_size,
+        batch_dim: tuple[int, int] = (0, 0),
+    ) -> None:
+        ...
         """
         params:
             * max_latency_in_ms: max_latency_in_ms for inbound tasks in milliseconds
             * max_batch_size: max batch size of inbound tasks
             * shared_sema: semaphore to limit concurrent outbound tasks
+            * get_batch_size: callable to get batch size from inputs
+            * batch_dim: tuple of (input_dim, output_dim) of batch
             * fallback: callable to return fallback result
         raises:
             * all possible exceptions the decorated function has
         """
-        self.max_latency_in_ms = max_latency_in_ms / 1000.0
+        self.max_latency = max_latency_in_ms / 1000.0
         self.fallback = fallback
-        self.optimizer = Optimizer(self.max_latency_in_ms)
+        self.optimizer = Optimizer(self.max_latency)
         self.max_batch_size = int(max_batch_size)
-        self.tick_interval = 0.001
 
         self._controller = None
-        self._queue: collections.deque[
-            Job
-        ] = collections.deque()  # TODO(bojiang): maxlen
+        self._queue: collections.deque[Job[T_IN, T_OUT]] = (
+            collections.deque()
+        )  # TODO(bojiang): maxlen
+        self.get_batch_size = get_batch_size
+        self.batch_dim = batch_dim
+        # at most 1 batch can be processed at the same time
         self._sema = shared_sema if shared_sema else NonBlockSema(1)
 
-    def shutdown(self):
+    def shutdown(self) -> None:
         if self._controller is not None:
             self._controller.cancel()
-        try:
-            while True:
-                fut = self._queue.pop().future
-                fut.cancel()
-        except IndexError:
-            pass
+        for job in self._queue:
+            job.future.cancel()
 
     @cached_property
-    def _loop(self):
+    def _loop(self) -> asyncio.AbstractEventLoop:
         return asyncio.get_event_loop()
 
     @cached_property
-    def _wake_event(self):
+    def _wake_event(self) -> asyncio.Condition:
         return asyncio.Condition()
 
     def __call__(
@@ -165,194 +175,113 @@ class CorkDispatcher:
         callback: t.Callable[
             [t.Sequence[T_IN]], t.Coroutine[None, None, t.Sequence[T_OUT]]
         ],
-    ) -> t.Callable[[T_IN], t.Coroutine[None, None, T_OUT]]:
-        self.callback = callback
+    ) -> t.Callable[[T_IN], t.Coroutine[None, None, T_OUT | None]]:
+        if is_async_callable(callback):
+            self.callback = callback
+        else:
+            self.callback = functools.partial(run_in_threadpool, callback)
 
         @functools.wraps(callback)
-        async def _func(data: t.Any) -> t.Any:
+        async def _func(data: T_IN) -> T_OUT:
             if self._controller is None:
                 self._controller = self._loop.create_task(self.controller())
             try:
                 r = await self.inbound_call(data)
             except asyncio.CancelledError:
-                return None if self.fallback is None else self.fallback()
+                return self.fallback()
             if isinstance(r, Exception):
                 raise r
             return r
 
         return _func
 
-    async def controller(self):
+    async def train_optimizer(
+        self,
+        num_required_reqs: int,
+        num_reqs_to_train: int,
+        training_batch_size: int,
+    ):
+        if self.max_batch_size < training_batch_size:
+            training_batch_size = self.max_batch_size
+
+        wait = 0.0
+        if training_batch_size > 1:
+            wait = min(
+                self.max_latency * 0.95,
+                # wait for at most approximated time to process 2n + 1 requests
+                (training_batch_size * 2 + 1) * self.optimizer.o_a + self.optimizer.o_b,
+            )
+
+        req_count = 0
+        try:
+            while req_count < num_reqs_to_train:
+                async with self._wake_event:  # block until there's any request in queue
+                    await self._wake_event.wait_for(self._queue.__len__)
+
+                n = len(self._queue)
+                now = time.time()
+                # the wait time of the first request
+                w0 = now - self._queue[0].enqueue_time
+
+                # only cancel requests if there are more than enough for training
+                if n > num_required_reqs - req_count and w0 >= self.max_latency:
+                    # we're being very conservative and only canceling requests if they have already timed out
+                    self._queue.popleft().future.cancel()
+                    continue
+                if training_batch_size > 1:  # only wait if batch_size
+                    a = self.optimizer.o_a
+                    b = self.optimizer.o_b
+
+                    if (
+                        n < training_batch_size
+                        and (training_batch_size * a + b) + w0 <= wait
+                    ):
+                        await asyncio.sleep(self.TICKET_INTERVAL)
+                        continue
+                if self._sema.is_locked():
+                    await asyncio.sleep(self.TICKET_INTERVAL)
+                    continue
+
+                req_count += 1
+                # call
+                self._sema.acquire()
+                inputs_info = tuple(self._get_inputs())
+                self._loop.create_task(self.outbound_call(inputs_info, training=True))
+        except Exception:
+            logger.exception("Error in training optimizer")
+
+    async def controller(self) -> None:
         """
         A standalone coroutine to wait/dispatch calling.
         """
-        logger.debug("Starting dispatcher optimizer training...")
-        # warm up the model
-        while self.optimizer.outbound_counter <= self.optimizer.N_SKIPPED_SAMPLE:
-            try:
-                async with self._wake_event:  # block until there's any request in queue
-                    await self._wake_event.wait_for(self._queue.__len__)
-
-                n = len(self._queue)
-                now = time.time()
-                w0 = now - self._queue[0].enqueue_time
-
-                # only cancel requests if there are more than enough for training
-                if (
-                    n
-                    > self.optimizer.N_SKIPPED_SAMPLE
-                    - self.optimizer.outbound_counter
-                    + 6
-                    and w0 >= self.max_latency_in_ms
-                ):
-                    # we're being very conservative and only canceling requests if they have already timed out
-                    self._queue.popleft().future.cancel()
-                    continue
-                # don't try to be smart here, just serve the first few requests
-                if self._sema.is_locked():
-                    await asyncio.sleep(self.tick_interval)
-                    continue
-
-                n_call_out = 1
-                # call
-                self._sema.acquire()
-                inputs_info = tuple(self._queue.pop() for _ in range(n_call_out))
-                self._loop.create_task(self.outbound_call(inputs_info))
-            except asyncio.CancelledError:
-                return
-            except Exception as e:  # pylint: disable=broad-except
-                logger.error(traceback.format_exc(), exc_info=e)
-
-        logger.debug("Dispatcher finished warming up model.")
-
-        while self.optimizer.outbound_counter <= self.optimizer.N_SKIPPED_SAMPLE + 1:
-            try:
-                # step 1: attempt to serve a single request immediately
-                async with self._wake_event:  # block until there's any request in queue
-                    await self._wake_event.wait_for(self._queue.__len__)
-
-                n = len(self._queue)
-                now = time.time()
-                w0 = now - self._queue[0].enqueue_time
-
-                # only cancel requests if there are more than enough for training
-                if n > 6 and w0 >= self.max_latency_in_ms:
-                    # we're being very conservative and only canceling requests if they have already timed out
-                    self._queue.popleft().future.cancel()
-                    continue
-                if self._sema.is_locked():
-                    await asyncio.sleep(self.tick_interval)
-                    continue
-
-                n_call_out = 1
-                # call
-                self._sema.acquire()
-                inputs_info = tuple(self._queue.pop() for _ in range(n_call_out))
-                self._loop.create_task(self.outbound_call(inputs_info))
-            except asyncio.CancelledError:
-                return
-            except Exception as e:  # pylint: disable=broad-except
-                logger.error(traceback.format_exc(), exc_info=e)
-
-        logger.debug("Dispatcher finished optimizer training request 1.")
-        self.optimizer.trigger_refresh()
-
-        if self.max_batch_size >= 2:
-            # we will attempt to keep the second request served within this time
-            step_2_wait = min(
-                self.max_latency_in_ms * 0.95,
-                5 * (self.optimizer.o_a + self.optimizer.o_b),
+        try:
+            logger.debug("Starting dispatcher optimizer training...")
+            # warm up the model
+            await self.train_optimizer(
+                self.optimizer.N_SKIPPED_SAMPLE, self.optimizer.N_SKIPPED_SAMPLE + 6, 1
             )
 
-            # step 2: attempt to serve 2 requests
-            while (
-                self.optimizer.outbound_counter <= self.optimizer.N_SKIPPED_SAMPLE + 2
-            ):
-                try:
-                    async with self._wake_event:  # block until there's any request in queue
-                        await self._wake_event.wait_for(self._queue.__len__)
+            logger.debug("Dispatcher finished warming up model.")
 
-                    n = len(self._queue)
-                    dt = self.tick_interval
-                    now = time.time()
-                    w0 = now - self._queue[0].enqueue_time
-                    a = self.optimizer.o_a
-                    b = self.optimizer.o_b
-
-                    # only cancel requests if there are more than enough for training
-                    if n > 5 and w0 >= self.max_latency_in_ms:
-                        # we're being very conservative and only canceling requests if they have already timed out
-                        self._queue.popleft().future.cancel()
-                        continue
-                    if n < 2 and (2 * a + b) + w0 <= step_2_wait:
-                        await asyncio.sleep(self.tick_interval)
-                        continue
-                    if self._sema.is_locked():
-                        await asyncio.sleep(self.tick_interval)
-                        continue
-
-                    n_call_out = min(n, 2)
-                    # call
-                    self._sema.acquire()
-                    inputs_info = tuple(self._queue.pop() for _ in range(n_call_out))
-                    self._loop.create_task(self.outbound_call(inputs_info))
-                except asyncio.CancelledError:
-                    return
-                except Exception as e:  # pylint: disable=broad-except
-                    logger.error(traceback.format_exc(), exc_info=e)
+            await self.train_optimizer(6, 1, 1)
+            self.optimizer.trigger_refresh()
+            logger.debug("Dispatcher finished optimizer training request 1.")
 
+            await self.train_optimizer(5, 1, 2)
+            self.optimizer.trigger_refresh()
             logger.debug("Dispatcher finished optimizer training request 2.")
+
+            await self.train_optimizer(3, 1, 3)
             self.optimizer.trigger_refresh()
-
-        if self.max_batch_size >= 3:
-            # step 3: attempt to serve 3 requests
-
-            # we will attempt to keep the second request served within this time
-            step_3_wait = min(
-                self.max_latency_in_ms * 0.95,
-                7 * (self.optimizer.o_a + self.optimizer.o_b),
-            )
-            while (
-                self.optimizer.outbound_counter <= self.optimizer.N_SKIPPED_SAMPLE + 3
-            ):
-                try:
-                    async with self._wake_event:  # block until there's any request in queue
-                        await self._wake_event.wait_for(self._queue.__len__)
-
-                    n = len(self._queue)
-                    dt = self.tick_interval
-                    now = time.time()
-                    w0 = now - self._queue[0].enqueue_time
-                    a = self.optimizer.o_a
-                    b = self.optimizer.o_b
-
-                    # only cancel requests if there are more than enough for training
-                    if n > 3 and w0 >= self.max_latency_in_ms:
-                        # we're being very conservative and only canceling requests if they have already timed out
-                        self._queue.popleft().future.cancel()
-                        continue
-                    if n < 3 and (3 * a + b) + w0 <= step_3_wait:
-                        await asyncio.sleep(self.tick_interval)
-                        continue
-
-                    n_call_out = min(n, 3)
-                    # call
-                    self._sema.acquire()
-                    inputs_info = tuple(self._queue.pop() for _ in range(n_call_out))
-                    self._loop.create_task(self.outbound_call(inputs_info))
-                except asyncio.CancelledError:
-                    return
-                except Exception as e:  # pylint: disable=broad-except
-                    logger.error(traceback.format_exc(), exc_info=e)
-
             logger.debug("Dispatcher finished optimizer training request 3.")
-            self.optimizer.trigger_refresh()
 
-        if self.optimizer.o_a + self.optimizer.o_b >= self.max_latency_in_ms:
-            logger.warning(
-                "BentoML has detected that a service has a max latency that is likely too low for serving. If many 503 errors are encountered, try raising the 'runner.max_latency' in your BentoML configuration YAML file."
-            )
-        logger.debug("Dispatcher optimizer training complete.")
+            if self.optimizer.o_a + self.optimizer.o_b >= self.max_latency:
+                logger.warning(
+                    "BentoML has detected that a service has a max latency that is likely too low for serving. If many 503 errors are encountered, try raising the 'runner.max_latency' in your BentoML configuration YAML file."
+                )
+            logger.debug("Dispatcher optimizer training complete.")
+        except Exception:  # pylint: disable=broad-except
+            logger.exception("Error training optimizer")
 
         while True:
             try:
@@ -360,10 +289,12 @@ class CorkDispatcher:
                     await self._wake_event.wait_for(self._queue.__len__)
 
                 n = len(self._queue)
-                dt = self.tick_interval
+                dt = self.TICKET_INTERVAL
                 decay = 0.95  # the decay rate of wait time
                 now = time.time()
+                # the wait time of the first request
                 w0 = now - self._queue[0].enqueue_time
+                # the wait time of the last request
                 wn = now - self._queue[-1].enqueue_time
                 a = self.optimizer.o_a
                 b = self.optimizer.o_b
@@ -371,71 +302,40 @@ class CorkDispatcher:
                 # the estimated latency of the first request if we began processing now
                 latency_0 = w0 + a * n + b
 
-                if n > 1 and latency_0 >= self.max_latency_in_ms:
+                if n > 1 and latency_0 >= self.max_latency:
                     self._queue.popleft().future.cancel()
                     continue
                 if self._sema.is_locked():
-                    if n == 1 and w0 >= self.max_latency_in_ms:
+                    if n == 1 and w0 >= self.max_latency:
                         self._queue.popleft().future.cancel()
                         continue
-                    await asyncio.sleep(self.tick_interval)
+                    await asyncio.sleep(self.TICKET_INTERVAL)
                     continue
                 if (
                     n < self.max_batch_size
                     and n * (wn + dt + (a or 0)) <= self.optimizer.wait * decay
                 ):
-                    n = len(self._queue)
-                    now = time.time()
-                    wn = now - self._queue[-1].enqueue_time
-                    latency_0 += dt
-
-                    # wait for additional requests to arrive
-                    await asyncio.sleep(self.tick_interval)
+                    await asyncio.sleep(self.TICKET_INTERVAL)
                     continue
-
-                n_call_out = 0
-                batch_size = 0
-                try:
-                    for input_info in self._queue:
-                        if (
-                            batch_size + input_info.data.sample.batch_size
-                            < self.max_batch_size
-                        ):
-                            n_call_out += 1
-                            batch_size += input_info.data.sample.batch_size
-                        else:
-                            break
-                except Exception as e:
-                    n_call_out = min(n, self.max_batch_size)
-                    logger.error(
-                        "error in batch-size aware batching, falling back to regular batching method",
-                        exc_info=e,
-                    )
-
                 # call
                 self._sema.acquire()
-                inputs_info = tuple(self._queue.pop() for _ in range(n_call_out))
+                inputs_info = tuple(self._get_inputs())
                 self._loop.create_task(self.outbound_call(inputs_info))
-            except asyncio.CancelledError:
-                return
-            except Exception as e:  # pylint: disable=broad-except
-                logger.error(traceback.format_exc(), exc_info=e)
-
-    async def inbound_call(self, data: Params[Payload]):
-        if data.sample.batch_size > self.max_batch_size:
-            raise RuntimeError(
-                f"batch of size {data.sample.batch_size} exceeds configured max batch size of {self.max_batch_size}."
-            )
+            except Exception:
+                logger.exception("Error processing batch requests")
 
+    async def inbound_call(self, data: T_IN) -> T_OUT | Exception:
         now = time.time()
-        future = self._loop.create_future()
+        future: asyncio.Future[T_OUT | Exception] = self._loop.create_future()
         input_info = Job(now, data, future)
         self._queue.append(input_info)
         async with self._wake_event:
             self._wake_event.notify_all()
         return await future
 
-    async def outbound_call(self, inputs_info: tuple[Job, ...]):
+    async def outbound_call(
+        self, inputs_info: tuple[Job[T_IN, T_OUT], ...], training: bool = False
+    ):
         _time_start = time.time()
         _done = False
         batch_size = len(inputs_info)
@@ -452,11 +352,10 @@ class CorkDispatcher:
             _done = True
             self.optimizer.log_outbound(
                 n=len(inputs_info),
-                wait=_time_start - inputs_info[-1].enqueue_time,
+                # fake wait as 0 for training requests
+                wait=_time_start - inputs_info[0].enqueue_time if not training else 0,
                 duration=time.time() - _time_start,
             )
-        except asyncio.CancelledError:
-            pass
         except Exception as e:  # pylint: disable=broad-except
             for input_info in inputs_info:
                 fut = input_info.future
@@ -470,3 +369,71 @@ class CorkDispatcher:
                     if not fut.done():
                         fut.cancel()
             self._sema.release()
+
+    def _get_inputs(
+        self, num_batches: int | None = None
+    ) -> t.Iterable[Job[T_IN, T_OUT]]:
+        if num_batches is None:
+            num_batches = self.max_batch_size
+        batch_size = 0
+        while len(self._queue) > 0 and batch_size < num_batches:
+            try:
+                next_batch_size = self.get_batch_size(self._queue[0].data)
+            except Exception:
+                logger.exception(
+                    "error in batch-size aware batching, falling back to regular batching method",
+                )
+                next_batch_size = 1
+            if batch_size + next_batch_size <= num_batches:
+                batch_size += next_batch_size
+                yield self._queue.popleft()
+            else:
+                job = self._queue.popleft()
+                chunk_size = num_batches - batch_size  # chunk_size < next_batch_size
+                first_job, second_job = self.split_job(
+                    job, [0, chunk_size, next_batch_size]
+                )
+                batch_size += chunk_size
+                self._queue.appendleft(second_job)
+                yield first_job
+
+    def split_job(
+        self, job: Job[T_IN, T_OUT], split_indexes: list[int]
+    ) -> tuple[Job[T_IN, T_OUT], Job[T_IN, T_OUT]]:
+        """Split the job into two child jobs and process them separately."""
+        from ..runner.container import AutoContainer
+
+        logger.debug(
+            "Splitting batch into two child batches with indexes: %s", split_indexes
+        )
+        split_batches = AutoContainer.batch_to_batches(
+            job.data, split_indexes, self.batch_dim[0]
+        )
+        assert len(split_batches) == 2
+        first_child = attr.evolve(
+            job, data=split_batches[0], future=self._loop.create_future()
+        )
+        second_child = attr.evolve(
+            job, data=split_batches[1], future=self._loop.create_future()
+        )
+
+        def child_done_callback(fut: asyncio.Future[T_OUT | Exception]):
+            if fut.cancelled():
+                job.future.cancel()
+                return
+            if job.future.done():
+                return
+            result = fut.result()
+            if isinstance(result, Exception):
+                job.future.set_result(result)
+                return
+            if first_child.future.done() and second_child.future.done():
+                result, _ = AutoContainer.batches_to_batch(
+                    [first_child.future.result(), second_child.future.result()],
+                    self.batch_dim[1],
+                )
+                job.future.set_result(result)
+
+        first_child.future.add_done_callback(child_done_callback)
+        second_child.future.add_done_callback(child_done_callback)
+        return first_child, second_child
