@@ -1,15 +1,18 @@
 from __future__ import annotations
 
-import typing as t
 import functools
+import inspect
+import typing as t
 from typing import TYPE_CHECKING
+from typing import AsyncGenerator
 
 import anyio
 
-from . import RunnerHandle
-from ..utils import Params
-from ..container import Payload
 from ..container import AutoContainer
+from ..container import Payload
+from ..utils import Params
+from ..utils import iterate_in_threadpool
+from . import RunnerHandle
 
 if TYPE_CHECKING:
     from ..runner import Runner
@@ -61,3 +64,16 @@ class LocalRunnerRef(RunnerHandle):
             *args,
             limiter=self._limiter,
         )
+
+    def async_stream_method(
+        self,
+        __bentoml_method: RunnerMethod[t.Any, P, R],
+        *args: P.args,
+        **kwargs: P.kwargs,
+    ) -> AsyncGenerator[R, None]:
+        generator = getattr(self._runnable, __bentoml_method.name)(*args, **kwargs)
+        # This allows user to define both sync and async generator
+        if inspect.isasyncgen(generator):
+            return generator
+        else:
+            return iterate_in_threadpool(generator)
