@@ -1,13 +1,13 @@
 from __future__ import annotations
 
-import os
-import sys
-import typing as t
 import logging
-from typing import TYPE_CHECKING
+import os
+import typing as t
 from functools import partial
+from typing import TYPE_CHECKING
 
 if TYPE_CHECKING:
+    from prometheus_client import Metric
 
     from ... import external_typing as ext
 
@@ -15,12 +15,7 @@ logger = logging.getLogger(__name__)
 
 
 class PrometheusClient:
-    def __init__(
-        self,
-        *,
-        multiproc: bool = True,
-        multiproc_dir: str | None = None,
-    ):
+    def __init__(self, *, multiproc: bool = True):
         """
         PrometheusClient is BentoML's own prometheus client that extends the official Python client.
 
@@ -33,39 +28,21 @@ class PrometheusClient:
            is called. This has to do with ``prometheus_client`` relies on ``PROMEHEUS_MULTIPROC_DIR``, which
            will be set by this client.
 
-        For API documentation, refer to https://docs.bentoml.org/en/latest/reference/metrics.html.
+        For API documentation, refer to https://docs.bentoml.com/en/latest/reference/metrics.html.
         """
-        if multiproc:
-            assert multiproc_dir is not None, "multiproc_dir must be provided"
-
         self.multiproc = multiproc
-        self.multiproc_dir: str | None = multiproc_dir
         self._registry = None
         self._imported = False
         self._pid: int | None = None
 
     @property
     def prometheus_client(self):
-        if self.multiproc and not self._imported:
-            # step 1: check environment
-            assert (
-                "prometheus_client" not in sys.modules
-            ), "prometheus_client is already imported, multiprocessing will not work properly"
-
-            assert (
-                self.multiproc_dir
-            ), f"Invalid prometheus multiproc directory: {self.multiproc_dir}"
-            assert os.path.isdir(self.multiproc_dir)
-
-            os.environ["PROMETHEUS_MULTIPROC_DIR"] = self.multiproc_dir
-
-        # step 2:
         import prometheus_client
-        import prometheus_client.parser
-        import prometheus_client.metrics
         import prometheus_client.exposition
+        import prometheus_client.metrics
         import prometheus_client.metrics_core
         import prometheus_client.multiprocess
+        import prometheus_client.parser
 
         self._imported = True
         return prometheus_client
@@ -79,9 +56,9 @@ class PrometheusClient:
         else:
             if self.multiproc:
                 assert self._pid is not None
-                assert (
-                    os.getpid() == self._pid
-                ), "The current process's different than the process which the prometheus client gets created"
+                assert os.getpid() == self._pid, (
+                    "The current process's different than the process which the prometheus client gets created"
+                )
 
         return self._registry
 
@@ -91,9 +68,9 @@ class PrometheusClient:
     def mark_process_dead(self) -> None:
         if self.multiproc:
             assert self._pid is not None
-            assert (
-                os.getpid() == self._pid
-            ), "The current process's different than the process which the prometheus client gets created"
+            assert os.getpid() == self._pid, (
+                "The current process's different than the process which the prometheus client gets created"
+            )
             self.prometheus_client.multiprocess.mark_process_dead(self._pid)
 
     def start_http_server(self, port: int, addr: str = "") -> None:
@@ -174,6 +151,10 @@ class PrometheusClient:
         """
         A Metric family and its samples.
 
-        This is a base class to be used by instrumentation client. Custom collectors should use ``bentoml.metrics.metrics_core.GaugeMetricFamily``, ``bentoml.metrics.metrics_core.CounterMetricFamily``, ``bentoml.metrics.metrics_core.SummaryMetricFamily`` instead.
+        This is a base class to be used by instrumentation client.
+        Custom collectors should use
+        ``prometheus_client.metrics_core.GaugeMetricFamily``,
+        ``prometheus_client.metrics_core.CounterMetricFamily``,
+        ``prometheus_client.metrics_core.SummaryMetricFamily`` instead.
         """
         return partial(self.prometheus_client.Metric, registry=self.registry)
