@@ -1,34 +1,31 @@
 from __future__ import annotations
 
 import typing as t
+from functools import lru_cache
 from http import HTTPStatus
 from typing import TYPE_CHECKING
-from functools import lru_cache
 
-from deepmerge.merger import Merger
-
-from bentoml.exceptions import NotFound
-from bentoml.exceptions import InvalidArgument
 from bentoml.exceptions import InternalServerError
+from bentoml.exceptions import InvalidArgument
+from bentoml.exceptions import NotFound
 
-from .utils import REF_PREFIX
-from .utils import exception_schema
-from .utils import exception_components_schema
 from ...types import LazyType
-from ...utils import bentoml_cattr
-from .specification import Tag
-from .specification import Info
+from ...utils import deep_merge
+from ...utils.cattr import bentoml_cattr
 from .specification import Contact
-from .specification import PathItem
-from .specification import Response
+from .specification import Info
 from .specification import MediaType
-from .specification import Operation
-from .specification import Reference
-from .specification import Components
 from .specification import OpenAPISpecification
+from .specification import Operation
+from .specification import PathItem
+from .specification import Reference
+from .specification import Response
+from .specification import Tag
+from .utils import REF_PREFIX
+from .utils import exception_components_schema
+from .utils import exception_schema
 
 if TYPE_CHECKING:
-
     from .. import Service
     from ..inference_api import InferenceAPI
 
@@ -51,17 +48,8 @@ APP_TAG = Tag(
     name="Service APIs", description="BentoML Service API endpoints for inference."
 )
 
-merger = Merger(
-    # merge dicts
-    [(dict, "merge")],
-    # override all other types
-    ["override"],
-    # override conflicting types
-    ["override"],
-)
 
-
-def make_api_path(api: InferenceAPI) -> str:
+def make_api_path(api: InferenceAPI[t.Any]) -> str:
     return api.route if api.route.startswith("/") else f"/{api.route}"
 
 
@@ -79,28 +67,27 @@ def make_infra_endpoints() -> dict[str, PathItem]:
     }
 
 
-def generate_service_components(svc: Service) -> Components:
+def generate_service_components(svc: Service) -> dict[str, t.Any]:
     components: dict[str, t.Any] = {}
     for api in svc.apis.values():
         api_components = {}
         input_components = api.input.openapi_components()
         if input_components:
-            merger.merge(api_components, input_components)
+            deep_merge(api_components, input_components)
         output_components = api.output.openapi_components()
         if output_components:
-            merger.merge(api_components, output_components)
+            deep_merge(api_components, output_components)
 
-        merger.merge(components, api_components)
+        deep_merge(components, api_components)
 
     # merge exception at last
-    merger.merge(components, {"schemas": exception_components_schema()})
-
-    return Components(**components)
+    return deep_merge(components, {"schemas": exception_components_schema()})
 
 
 def generate_spec(svc: Service, *, openapi_version: str = "3.0.2"):
     """Generate a OpenAPI specification for a service."""
     mounted_app_paths = {}
+    schema_components = {}
 
     for app, _, _ in svc.mount_apps:
         if LazyType["fastapi.FastAPI"]("fastapi.FastAPI").isinstance(app):
@@ -119,10 +106,15 @@ def generate_spec(svc: Service, *, openapi_version: str = "3.0.2"):
                 }
             )
 
+            if "components" in openapi:
+                deep_merge(schema_components, openapi["components"])
+
+    deep_merge(schema_components, generate_service_components(svc))
+
     return OpenAPISpecification(
         openapi=openapi_version,
         tags=[APP_TAG, INFRA_TAG],
-        components=generate_service_components(svc),
+        components=schema_components,
         info=Info(
             title=svc.name,
             description=svc.doc,
