@@ -34,12 +34,7 @@ class GenerateLatestMiddleware:
 @click.command()
 @click.option("--fd", type=click.INT, required=True)
 @click.option("--backlog", type=click.INT, default=2048)
-@click.option(
-    "--prometheus-dir",
-    type=click.Path(exists=True),
-    help="Required by prometheus to pass the metrics in multi-process mode",
-)
-def main(fd: int, backlog: int, prometheus_dir: str | None):
+def main(fd: int, backlog: int):
     """
     Start a standalone Prometheus server to use with gRPC.
     \b
@@ -51,24 +46,21 @@ def main(fd: int, backlog: int, prometheus_dir: str | None):
 
     import psutil
     import uvicorn
-    from starlette.middleware import Middleware
+    from a2wsgi import WSGIMiddleware
     from starlette.applications import Starlette
-    from starlette.middleware.wsgi import WSGIMiddleware  # TODO: a2wsgi
+    from starlette.middleware import Middleware
 
-    from bentoml._internal.log import configure_server_logging
-    from bentoml._internal.context import component_context
     from bentoml._internal.configuration import get_debug_mode
     from bentoml._internal.configuration.containers import BentoMLContainer
+    from bentoml._internal.context import server_context
+    from bentoml._internal.log import configure_server_logging
 
-    component_context.component_type = "prom_server"
+    server_context.service_type = "prom_server"
 
     configure_server_logging()
 
     BentoMLContainer.development_mode.set(False)
     metrics_client = BentoMLContainer.metrics_client.get()
-    if prometheus_dir is not None:
-        BentoMLContainer.prometheus_multiproc_dir.set(prometheus_dir)
-
     # create a ASGI app that wraps around the default HTTP prometheus server.
     prom_app = Starlette(
         debug=get_debug_mode(), middleware=[Middleware(GenerateLatestMiddleware)]
